@@ -2,7 +2,7 @@
 #
 #    Limited command Shell (lshell)
 #  
-#    $Id: lshell.py,v 1.51 2009/11/30 22:40:05 ghantoos Exp $
+#    $Id: lshell.py,v 1.65 2010/05/04 19:50:29 ghantoos Exp $
 #
 #    Copyright (C) 2008-2009 Ignace Mouzannar (ghantoos) <ghantoos@ghantoos.org>
 #
@@ -41,7 +41,7 @@ import grp
 import time
 
 __author__ = "Ignace Mouzannar (ghantoos) <ghantoos@ghantoos.org>"
-__version__ = "0.9.8"
+__version__ = "0.9.12"
 
 # Required config variable list per user
 required_config = ['allowed', 'forbidden', 'warning_counter'] 
@@ -60,7 +60,10 @@ else:
 config_file = conf_prefix + '/etc/lshell.conf'
 
 # history file
-hisory_file = ".lhistory"
+history_file = ".lhistory"
+
+# lock_file
+lock_file = ".lshell_lock"
 
 # help text
 usage = """Usage: lshell [OPTIONS]
@@ -82,7 +85,8 @@ class ShellCmd(cmd.Cmd, object):
     """ Main lshell CLI class
     """
 
-    def __init__(self, userconf, stdin=None, stdout=None, stderr=None):
+    def __init__(self, userconf, stdin=None, stdout=None, stderr=None,         \
+                                    g_cmd=None, g_line=None):
         if stdin is None:
             self.stdin = sys.stdin
         else:
@@ -98,18 +102,21 @@ class ShellCmd(cmd.Cmd, object):
 
         self.conf = userconf
         self.log = self.conf['logpath']
+
         # Set timer
         if self.conf['timer'] > 0: self.mytimer(self.conf['timer'])
         self.identchars = self.identchars + '+./-'
         self.log.error('Logged in')
-        self.history = os.path.normpath(self.conf['home_path']) + '/'          \
-                                                                + hisory_file
         cmd.Cmd.__init__(self)
         self.prompt = self.conf['username'] + ':~$ '
-        self.intro = intro
+        self.intro = self.conf['intro']
+
+        # initialize cli variables
+        self.g_cmd = g_cmd
+        self.g_line = g_line
 
     def __getattr__(self, attr):
-        """This method actually takes care of all the called method that are   \
+        """ This method actually takes care of all the called method that are  \
         not resolved (i.e not existing methods). It actually will simulate     \
         the existance of any method    entered in the 'allowed' variable list. \
 
@@ -122,7 +129,7 @@ class ShellCmd(cmd.Cmd, object):
             if self.g_cmd == 'EOF':
                 self.stdout.write('\n')
             sys.exit(0)
-        if self.check_secure(self.g_line) == 1: 
+        if self.check_secure(self.g_line, self.conf['strict']) == 1: 
             return object.__getattribute__(self, attr)
         if self.check_path(self.g_line) == 1:
             return object.__getattribute__(self, attr)
@@ -134,28 +141,47 @@ class ShellCmd(cmd.Cmd, object):
             if type(self.conf['aliases']) == dict:
                 self.g_line = get_aliases(self.g_line, self.conf['aliases'])
             self.log.info('CMD: "%s"' %self.g_line)
-            if self.g_cmd in ['cd']:
-                if len(self.g_arg) >= 1:
-                    try:
-                        os.chdir(os.path.realpath(self.g_arg))
-                        self.updateprompt(os.getcwd())
-                    except OSError, (ErrorNumber, ErrorMessage):
-                        print "lshell: %s:" %self.g_arg, ErrorMessage
-                else: 
-                    os.chdir(self.conf['home_path'])
-                    self.updateprompt(os.getcwd())
+            if self.g_cmd == 'cd':
+                self.cd()
+            # list all allowed path
+            elif self.g_cmd == 'lpath':
+                self.lpath()
             else:
                 os.system(self.g_line)
-        elif self.g_cmd not in ['', '?', 'help', None] : 
-            if self.conf['strict'] == 1:
-                self.counter_update('command')
-            else:
-                self.log.warn('INFO: unknown syntax -> "%s"' %self.g_line)
-                self.stderr.write('*** unknown syntax: %s\n' %self.g_cmd)
+        elif self.g_cmd not in ['', '?', 'help', None]: 
+            self.log.warn('INFO: unknown syntax -> "%s"' %self.g_line)
+            self.stderr.write('*** unknown syntax: %s\n' %self.g_cmd)
         self.g_cmd, self.g_arg, self.g_line = ['', '', ''] 
         return object.__getattribute__(self, attr)
 
-    def check_secure(self, line):
+    def lpath(self):
+        """ lists allowed and forbidden path
+        """
+        if self.conf['path'][0]:
+            sys.stdout.write("Allowed:\n")
+            for path in self.conf['path'][0].split('|'):
+                if path:
+                    sys.stdout.write(" %s\n" %path[:-2])
+        if self.conf['path'][1]:
+            sys.stdout.write("Denied:\n")
+            for path in self.conf['path'][1].split('|'):
+                if path:
+                    sys.stdout.write(" %s\n" %path[:-2])
+
+    def cd(self):
+        """ implementation of the "cd" command
+        """
+        if len(self.g_arg) >= 1:
+            try:
+                os.chdir(os.path.realpath(self.g_arg))
+                self.updateprompt(os.getcwd())
+            except OSError, (ErrorNumber, ErrorMessage):
+                sys.stdout.write("lshell: %s: %s\n" %(self.g_arg, ErrorMessage))
+        else:
+            os.chdir(self.conf['home_path'])
+            self.updateprompt(os.getcwd())
+
+    def check_secure(self, line, strict=None, ssh=None):
         """This method is used to check the content on the typed command.      \
         Its purpose is to forbid the user to user to override the lshell       \
         command restrictions. 
@@ -165,20 +191,80 @@ class ShellCmd(cmd.Cmd, object):
         A warining counter has been added, to kick out of lshell a user if he  \
         is warned more than X time (X beeing the 'warning_counter' variable).
         """
+
         for item in self.conf['forbidden']:
-            if item in line:
-                self.counter_update('synthax')
-                return 1
+            # allow '&&' and '||' even if singles are forbidden
+            if item in ['&', '|']:
+                if re.findall("[^\%s]\%s[^\%s]" %(item, item, item), line):
+                    if not ssh:
+                        self.counter_update('syntax')
+                    return 1
+            else:
+                if item in line:
+                    if not ssh:
+                        self.counter_update('syntax')
+                    return 1
+
+        returncode = 0
+        # check if the line contains $(foo) executions, and check them
+        executions = re.findall('\$\([^)]+[)]', line)
+        for item in executions:
+            returncode += self.check_path(item[2:-1].strip())
+            returncode += self.check_secure(item[2:].strip(), strict=1)
+
+        # check fot executions using back quotes '`'
+        executions = re.findall('\`[^`]+[`]', line)
+        for item in executions:
+            returncode += self.check_secure(item[1:-1].strip(), strict=1)
+
+        # check if the line contains ${foo=bar}, and check them
+        curly = re.findall('\$\{[^}]+[}]', line)
+        for item in curly:
+            # split to get get variable only, and remove last character "}"
+            variable = re.split('=|\+|\?|\-', item, 1)
+            returncode += self.check_path(variable[1][:-1])
+            
+        # if unknown commands where found, return 1 and don't execute the line
+        if returncode > 0:
+            return 1
+        # in case the $(foo) or `foo` command passed the above tests
+        elif line.startswith('$(') or line.startswith('`'):
+            return 0
 
         # in case ';', '|' or '&' are not forbidden, check if in line
         lines = re.split('&|\||;', line)
+        # remove trailing parenthesis
+        line = re.sub('\)$', '', line)
         for sperate_line in lines:
-            command = sperate_line.strip().split(' ')[0]
-            if command not in self.conf['allowed'] and command:
-                if len(lines) > 1:
-                    self.counter_update('command', line)
+            splitcmd = sperate_line.strip().split(' ')
+            command = splitcmd[0]
+            if len(splitcmd) > 1:
+                cmdargs = splitcmd
+            else: cmdargs = None
+
+            # in case of a sudo command, check in sudo_commands list if allowed
+            if command == 'sudo':
+                if cmdargs[1] not in self.conf['sudo_commands'] and cmdargs:
+                    if self.conf['strict'] == 1:
+                        if not ssh:
+                            self.counter_update('command')
+                    else:
+                        self.log.critical('*** forbidden sudo -> %s'   \
+                                                % line )
                     return 1
-                return 0
+            # if over SSH, replaced allowed list with the one of overssh
+            if ssh:
+                self.conf['allowed'] = self.conf['overssh']
+            
+            # for all other commands check in allowed list
+            if command not in self.conf['allowed'] and command:
+                if strict:
+                    if not ssh:
+                        self.counter_update('command', line)
+                else:
+                    self.log.critical('*** unknown command: %s' %command)
+                return 1
+        return 0
          
     def counter_update(self, messagetype, path=None):
         """ Update the warning_counter, log and display a warning to the user
@@ -191,17 +277,17 @@ class ShellCmd(cmd.Cmd, object):
         if self.conf['warning_counter'] <= 0: 
             self.log.critical('*** forbidden %s -> "%s"'                       \
                                                   % (messagetype ,line))
-            self.log.critical('- Kicked out -')
+            self.log.critical('*** Kicked out')
             sys.exit(1)
         else:
             self.log.critical('*** forbidden %s -> "%s"'                       \
                                                   % (messagetype ,line))
-            self.stderr.write('*** You have %s joker(s) left,'                 \
+            self.stderr.write('*** You have %s warning(s) left,'               \
                                 ' before getting kicked out.\n'                \
                                 %(self.conf['warning_counter']-1))
             self.stderr.write('This incident has been reported.\n')
 
-    def check_path(self, line, completion=0):
+    def check_path(self, line, completion=None, ssh=None):
         """ Check if a path is entered in the line. If so, it checks if user   \
         are allowed to see this path. If user is not allowed, it calls         \
         self.counter_update. I case of completion, it only returns 0 or 1.
@@ -210,27 +296,53 @@ class ShellCmd(cmd.Cmd, object):
         allowed_path_re = str(self.conf['path'][0])
         denied_path_re = str(self.conf['path'][1][:-1])
 
-        if completion == 1:
-            line = line.strip().split()
-        else:
-            line = line.strip().split()[1:]
-
+        line = line.strip().split()
         for item in line:
+            # remove potential quotes
+            try:
+                item = eval(item)
+            except:
+                pass
+            # if item has been converted to somthing other than a string
+            # or an int, reconvert it to a string
+            if type(item) not in ['str', 'int']:
+                item = str(item)
+            # replace "~" with home path
+            item = os.path.expanduser(item)
+            # if contains a shell variable
+            if re.findall('\$|\*|\?',item):
+                # remove quotes if available
+                item = re.sub("\"|\'","",item)
+                # expand shell variables (method 1)
+                #for var in re.findall(r'\$(\w+|\{[^}]*\})', item):
+                #    # get variable value (if defined)
+                #    if os.environ.has_key(var):
+                #        value = os.environ[var]
+                #    else: value = ''
+                #    # replace the variable
+                #    item = re.sub('\$%s|\${%s}' %(var, var), value, item)
+                # expand shell variables and wildcards using "echo"
+                # i know, this a bit nasty...
+                cin, cout = os.popen2('`which echo` %s' % item)
+                item = cout.readlines()[0].split(' ')[0].strip()
+                item = os.path.expandvars(item)
             tomatch = os.path.realpath(item)
-            if os.path.isdir(tomatch): tomatch += '/'
+            if os.path.isdir(tomatch) and tomatch[-1] != '/': tomatch += '/'
             match_allowed = re.findall(allowed_path_re, tomatch)
             if denied_path_re: 
                 match_denied = re.findall(denied_path_re, tomatch)
             else: match_denied = None
             if not match_allowed or match_denied:
-                if completion == 0:
-                    self.counter_update('path', tomatch)
+                if not completion:
+                    if not ssh:
+                        self.counter_update('path', tomatch)
                 return 1
-        if completion == 0:
-            if not re.findall(allowed_path_re, os.getcwd()+'/'): 
-                self.counter_update('path', os.getcwd())
-                os.chdir(self.conf['home_path'])
-                self.updateprompt(os.getcwd())
+        if not completion:
+            if not re.findall(allowed_path_re, os.getcwd()+'/'):
+                if not ssh:
+                    self.counter_update('path', os.getcwd())
+                    os.chdir(self.conf['home_path'])
+                    self.updateprompt(os.getcwd())
                 return 1
         return 0
 
@@ -255,12 +367,13 @@ class ShellCmd(cmd.Cmd, object):
         self.preloop()
         if self.use_rawinput and self.completekey:
             try:
-                readline.read_history_file(self.history)
+                readline.read_history_file(self.conf['history_file'])
+                readline.set_history_length(self.conf['history_size'])
             except IOError:
                 # if history file does not exist
                 try:
-                    open(self.history, 'w').close()
-                    readline.read_history_file(self.history)
+                    open(self.conf['history_file'], 'w').close()
+                    readline.read_history_file(self.conf['history_file'])
                 except IOError:
                     pass
             self.old_completer = readline.get_completer()
@@ -269,8 +382,8 @@ class ShellCmd(cmd.Cmd, object):
         try:
             if intro is not None:
                 self.intro = intro
-            if self.intro:
-                self.stdout.write(str(self.intro)+"\n")
+            if self.conf['intro']:
+                self.stdout.write(str(self.conf['intro'])+"\n")
             stop = None
             while not stop:
                 if self.cmdqueue:
@@ -304,10 +417,10 @@ class ShellCmd(cmd.Cmd, object):
                 except ImportError:
                     pass
             try:
-                readline.write_history_file(self.history)
+                readline.write_history_file(self.conf['history_file'])
             except IOError:
                 self.log.error('WARN: couldn\'t write history '                \
-                                                'to file %s\n' %self.history)
+                                   'to file %s\n' % self.conf['history_file'])
 
     def complete(self, text, state):
         """Return the next possible completion for 'text'.
@@ -322,7 +435,9 @@ class ShellCmd(cmd.Cmd, object):
             stripped = len(origline) - len(line)
             begidx = readline.get_begidx() - stripped
             endidx = readline.get_endidx() - stripped
-            if line.split(' ')[0] in self.conf['allowed']:
+            if line.split(' ')[0] == 'sudo' and len(line.split(' ')) <= 2:
+                compfunc = self.completesudo
+            elif line.split(' ')[0] in self.conf['allowed']:
                 compfunc = self.completechdir
             elif begidx > 0:
                 cmd, args, foo = self.parseline(line)
@@ -361,10 +476,18 @@ class ShellCmd(cmd.Cmd, object):
             names.append('do_' + command)
         return [a[3:] for a in names if a.startswith(dotext)]
 
+    def completesudo(self, text, line, begidx, endidx):
+        """ complete sudo command """
+        return [a for a in self.conf['sudo_commands'] if a.startswith(text)]
+
     def completechdir(self, text, line, begidx, endidx):
+        """ complete directories """
         toreturn = []
+        tocomplete = line.split()[-1]
+        # replace "~" with home path
+        tocomplete = re.sub('^~', self.conf['home_path'], tocomplete)
         try:
-            directory = os.path.realpath(line.split()[-1])
+            directory = os.path.realpath(tocomplete)
         except: 
             directory = os.getcwd()
 
@@ -495,7 +618,7 @@ class CheckConfig:
         """ This method checks the usage. lshell.py must be called with a      \
         configuration file.
         If no configuration file is specified, it will set the configuration   \
-        file path to /etc/lshell.conf.
+        file path to /etc/lshell.confelf.conf['allowed'].append('exit')
         """
         # uncomment the following to set the -c/--config as mandatory argument
         #if '-c' not in arguments and '--config' not in arguments:
@@ -531,6 +654,11 @@ class CheckConfig:
         if conf.has_key('logpath'): args += ['--log', conf['logpath']]
         os.environ['LSHELL_ARGS'] = str(args)
 
+        # if lshell is invoked using shh autorized_keys file e.g.
+        # command="/usr/bin/lshell", ssh-dss ....
+        if os.environ.has_key('SSH_ORIGINAL_COMMAND'):
+            conf['ssh'] = os.environ['SSH_ORIGINAL_COMMAND']
+
         return conf, args
 
     def usage(self):
@@ -664,10 +792,10 @@ class CheckConfig:
         # get user configuration if any
         self.get_config_sub(self.user)
 
-        #print self.conf_raw
-
     def get_config_sub(self, section):
-        """ self.get_config sub function """
+        """ this function is used to interpret the configuration +/-, 
+            'all' etc.
+        """
         if self.config.has_section(section):
             for item in self.config.items(section):
                 key = item[0]
@@ -688,7 +816,6 @@ class CheckConfig:
                             liste = ['', '']
                             for path in eval(stuff):
                                 liste[0] += os.path.realpath(path) + '/.*|'
-                            liste[0] = liste[0]
                             self.conf_raw.update({key:str(liste)})
                         elif stuff and type(eval(stuff)) is list:
                             self.conf_raw.update({key:stuff})
@@ -699,7 +826,6 @@ class CheckConfig:
                     liste = ['', '']
                     for path in self.myeval(value, 'path'):
                         liste[0] += os.path.realpath(path) + '/.*|'
-                    liste[0] = liste[0]
                     self.conf_raw.update({key:str(liste)})
                 else:
                     self.conf_raw.update(dict([item]))
@@ -726,7 +852,6 @@ class CheckConfig:
             if key == 'path':
                 for path in sublist:
                     liste[1] += os.path.realpath(path) + '/.*|'
-                liste[1] = liste[1]
             else:
                 for item in sublist:
                     if item in liste:
@@ -735,7 +860,6 @@ class CheckConfig:
                         self.log.error("CONF: -['%s'] ignored in '%s' list."   \
                                                                  %(item,key))
         return {key:str(liste)}
-            
 
     def expand_all(self):
         """ expand allowed, if set to 'all'
@@ -796,18 +920,29 @@ class CheckConfig:
 
         for item in ['allowed',
                     'forbidden',
+                    'sudo_commands',
                     'warning_counter',
                     'timer',
                     'scp',
+                    'scp_upload',
+                    'scp_download',
                     'sftp',
                     'overssh',
                     'strict',
-                    'aliases']:
+                    'aliases',
+                    'history_size']:
             try:
                 self.conf[item] = self.myeval(self.conf_raw[item], item)
             except KeyError:
-                if item in ['allowed', 'overssh']:
+                if item in ['allowed', 'overssh', 'sudo_commands']:
                     self.conf[item] = []
+                elif item in ['history_size']:
+                    self.conf[item] = -1
+                # default scp is allowed
+                elif item in ['scp_upload', 'scp_download']:
+                    self.conf[item] = 1
+                elif item in ['aliases']:
+                    self.conf[item] = {}
                 else:
                     self.conf[item] = 0
             except TypeError:
@@ -850,6 +985,16 @@ class CheckConfig:
             except TypeError:
                 self.log.error('CONF: scpforce must be a string!')
 
+        if self.conf_raw.has_key('intro'):
+            self.conf['intro'] = self.myeval(self.conf_raw['intro'])
+        else:
+            self.conf['intro'] = intro
+
+        # check if user account if locked
+        if self.conf_raw.has_key('lock_counter'):
+            self.conf['lock_counter'] = self.conf_raw['lock_counter']
+            self.account_lock(self.user, self.conf['lock_counter'], 1)
+
         if os.path.isdir(self.conf['home_path']):
             os.chdir(self.conf['home_path'])
         else:
@@ -857,9 +1002,40 @@ class CheckConfig:
                                                     % self.conf['home_path'])
             sys.exit(0)
 
+        if self.conf_raw.has_key('history_file'):
+            try:
+                self.conf['history_file'] =                                    \
+                               eval(self.conf_raw['history_file'].replace(     \
+                                                  "%u", self.conf['username']))
+            except:
+                self.log.error('CONF: history file error: %s'                  \
+                                                % self.conf['history_file'])
+        else:
+            self.conf['history_file'] = history_file
+
+        if not self.conf['history_file'].startswith('/'):
+            self.conf['history_file'] = "%s/%s" % ( self.conf['home_path'],    \
+                                                    self.conf['history_file'])
+
         os.environ['PATH'] = os.environ['PATH'] + self.conf['env_path']
 
+        # append default commands to allowed list
         self.conf['allowed'].append('exit')
+        self.conf['allowed'].append('lpath')
+        self.conf['allowed'].append('clear')
+
+        if self.conf['sudo_commands']:
+            self.conf['allowed'].append('sudo')
+
+    def account_lock(self, user, lock_counter, check=None):
+        """ check if user account is locked, in which case, exit """
+        ### TODO ###
+        # check if account is locked
+        if check:
+            pass
+        # increment account lock
+        else:
+            pass
 
     def check_scp_sftp(self):
         """ This method checks if the user is trying to SCP a file onto the    \
@@ -881,11 +1057,11 @@ class CheckConfig:
                         self.log.error('*** forbidden SFTP connection')
                         sys.exit(0)
 
-                # case no tty is given to the session (sftp, scp, cmd over ssh)
-                if self.conf['ssh'].find('&') > -1                             \
-                            or self.conf['ssh'].find(';') > -1                 \
-                            or self.conf['ssh'].find('|') > -1 :
-                    self.ssh_warn('char over SSH', self.conf['ssh'])
+                # initialise cli session
+                cli = ShellCmd(self.conf, None, None, None, None,              \
+                                                            self.conf['ssh'])
+                if cli.check_path(self.conf['ssh'], None, ssh=1):
+                    self.ssh_warn('path over SSH', self.conf['ssh'])
 
                 # check path first
                 allowed_path_re = str(self.conf['path'][0])
@@ -903,20 +1079,37 @@ class CheckConfig:
                 if self.conf['ssh'].startswith('scp '):
                     if self.conf['scp'] is 1 or 'scp' in self.conf['overssh']: 
                         if ' -f ' in self.conf['ssh']:
-                            self.log.error('SCP: GET "%s"' %self.conf['ssh'])
+                            # case scp download is allowed
+                            if self.conf['scp_download']:
+                                self.log.error('SCP: GET "%s"' \
+                                                            % self.conf['ssh'])
+                            # case scp download is forbidden
+                            else:
+                                self.log.error('SCP: download forbidden: "%s"' \
+                                                            % self.conf['ssh'])
+                                sys.exit(0)
                         elif ' -t ' in self.conf['ssh']:
-                            if self.conf.has_key('scpforce'):
-                                cmdsplit = self.conf['ssh'].split(' ')
-                                scppath = os.path.realpath(cmdsplit[-1])
-                                forcedpath = os.path.realpath(self.conf
-                                                                ['scpforce'])
-                                if scppath != forcedpath:
-                                    self.log.error('SCP: forced SCP directory:'
-                                                          + ' %s' %scppath)
-                                    cmdsplit.pop(-1)
-                                    cmdsplit.append(forcedpath)
-                                    self.conf['ssh'] = string.join(cmdsplit)
-                            self.log.error('SCP: PUT "%s"' %self.conf['ssh'])
+                            # case scp upload is allowed
+                            if self.conf['scp_upload']:
+                                if self.conf.has_key('scpforce'):
+                                    cmdsplit = self.conf['ssh'].split(' ')
+                                    scppath = os.path.realpath(cmdsplit[-1])
+                                    forcedpath = os.path.realpath(self.conf
+                                                                   ['scpforce'])
+                                    if scppath != forcedpath:
+                                        self.log.error('SCP: forced SCP '      \
+                                                       + 'directory: %s'       \
+                                                                    %scppath)
+                                        cmdsplit.pop(-1)
+                                        cmdsplit.append(forcedpath)
+                                        self.conf['ssh'] = string.join(cmdsplit)
+                                self.log.error('SCP: PUT "%s"'                 \
+                                                        %self.conf['ssh'])
+                            # case scp upload is forbidden
+                            else:
+                                self.log.error('SCP: upload forbidden: "%s"'   \
+                                                            % self.conf['ssh']) 
+                                sys.exit(0)
                         os.system(self.conf['ssh'])
                         self.log.error('SCP disconnect')
                         sys.exit(0)
@@ -924,12 +1117,20 @@ class CheckConfig:
                         self.ssh_warn('SCP connection', self.conf['ssh'], 'scp')
 
                 # check if command is in allowed overssh commands
-                elif self.conf['ssh'].strip().split()[0] in                    \
-                                                       self.conf['overssh']:
-                    self.log.error('Over SSH: "%s"' %self.conf['ssh'])
+                elif self.conf['ssh']:
+                    # replace aliases
                     self.conf['ssh'] = get_aliases(self.conf['ssh'],           \
                                                          self.conf['aliases'])
-                    os.system(self.conf['ssh'])
+                    # if command is not "secure", exit
+                    if cli.check_secure(self.conf['ssh'], strict=1, ssh=1):
+                        self.ssh_warn('char/command over SSH', self.conf['ssh'])
+                    # else
+                    self.log.error('Over SSH: "%s"' %self.conf['ssh'])
+                    # if command is "help"
+                    if self.conf['ssh'] == "help":
+                        cli.do_help(None)
+                    else:
+                        os.system(self.conf['ssh'])
                     self.log.error('Exited')
                     sys.exit(0)
 
@@ -949,6 +1150,7 @@ class CheckConfig:
         else:
             self.log.critical('*** forbidden %s: "%s"' %(message, command))
         self.stderr.write('This incident has been reported.\n')
+        self.log.error('Exited')
         sys.exit(0)
 
     def check_passwd(self):
@@ -990,19 +1192,16 @@ class LshellTimeOut(Exception):
 def get_aliases(line, aliases):
     """ Replace all configured aliases in the line
     """
-    line = re.sub('\s+', ' ', line.strip())
-    for char in [';', '&', '|']:
-        # remove spaces after char
-        line = line.replace('%s ' %char, '%s' %char)
-        # double char
-        line = line.replace('%s' %char, '%s%s' %(char, char))
-
     for item in aliases.keys():
-        line = re.sub('(^|;|&|\|)%s' %item, aliases[item], line)
+        reg = '(^| |;)%s([ ;&\|]+|$)' % item
+        while re.findall(reg, line):
+            beforecommand = re.findall(reg, line)[0][0]
+            aftercommand = re.findall(reg, line)[0][1]
+            line = re.sub(reg, "%s%s%s" % (beforecommand, aliases[item],       \
+                                                     aftercommand), line, 1)
     for char in [';', '&', '|']:
         # remove all remaining double char
         line = line.replace('%s%s' %(char, char), '%s' %char)
-
     return line
 
 def main():
@@ -1028,5 +1227,3 @@ def main():
 
 if __name__ == '__main__':
     main()
-
-
